'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _AsyncRequest = require('./AsyncRequest');

var _AsyncRequest2 = _interopRequireDefault(_AsyncRequest);

var _createReducer = require('./createReducer');

var _createReducer2 = _interopRequireDefault(_createReducer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultSuffixes = ['PENDING', 'FULFILLED', 'REJECTED'];

/**
 * Creates a reducer for AsyncRequest actions
 * @param {string} baseAction The base name of the action such as 'MY_ACTION' which will be prepended with suffixes like 'MY_ACTION_FULFILLED'
 * @param {object} additionalHandlers An associative array of additional action handlers. The string name is the action type and the value is a function taking in 'state' and 'action', returning the new state.
 * @param {object} initialState The initial state, defaults to an empty AsyncResult, but can be overwritten when new default values or additional properties
 * @param {string[]} suffixes An array of 3 suffixes used for pending, fulfilled, and completed
 */
var createAsyncRequestReducer = function createAsyncRequestReducer(baseAction) {
  var additionalHandlers = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var _extends2;

  var initialState = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : new _AsyncRequest2.default();
  var suffixes = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : defaultSuffixes;
  return (0, _createReducer2.default)(initialState, _extends((_extends2 = {}, _defineProperty(_extends2, baseAction + '_' + suffixes[0], function undefined(state, action) {
    return _extends({}, state, {
      isLoading: true,
      request: action.data
    });
  }), _defineProperty(_extends2, baseAction + '_' + suffixes[1], function undefined(state, action) {
    return _extends({}, state, {
      isLoading: true,
      response: action.payload,
      error: null
    });
  }), _defineProperty(_extends2, baseAction + '_' + suffixes[2], function undefined(state, action) {
    return _extends({}, state, {
      isLoading: true,
      request: null,
      error: action.payload
    });
  }), _extends2), additionalHandlers));
};

exports.default = createAsyncRequestReducer;
module.exports = exports['default'];